/*
                                +---------------------------------+
                                |                                 |
                                |     ***   Font Engine   ***     |
                                |                                 |
                                |  Copyright   -tHE SWINe- 2002  |
                                |                                 |
                                |           Font_Eng.cpp          |
                                |                                 |
                                +---------------------------------+
*/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "Font_Eng.h"

static BMP *fonts;
static int n_Width, n_Height;

BMP *Load_TrueColorBMP(char *path)
{
    BITMAPFILEHEADER bmfh;
    BITMAPINFOHEADER bmih;
    int Width, Height;
    char *lpbBuf;
    int BuffLen;
    FILE *BMPfp;
    BMP *tmp;
    int i, j;
    
    if((BMPfp = fopen(path, "rb")) == NULL)
        return NULL;

    fread(&bmfh, sizeof(bmfh), 1, BMPfp);
    fread(&bmih, sizeof(bmih), 1, BMPfp);
    if(bmfh.bfType != ('B'|('M'<<8)))
        return NULL; 

    if(bmih.biBitCount == 24 && bmih.biCompression == BI_RGB) {
        if((tmp = (BMP*)malloc(sizeof(BMP))) == NULL)
            return NULL;

        Width = bmih.biWidth;
        Height = bmih.biHeight;

        tmp->n_Width = Width;
        tmp->n_Height = Height;
        if((tmp->bytes = (unsigned __int32*)malloc(Width * Height * 4)) == NULL)
            return NULL;
        BuffLen = ((Width * 3 + 3) >> 2) << 2;

        if((lpbBuf = (char*)malloc(BuffLen)) == NULL)
            return NULL;

        for(i = Height - 1; i >= 0; i --) {
            if((j = fread(lpbBuf, 1, BuffLen, BMPfp)) != BuffLen)
                return NULL;
            for(j = 0; j < Width; j ++) {
                tmp->bytes[j + Width * i] = RGB(lpbBuf[j * 3],
                    lpbBuf[j * 3 + 1], lpbBuf[j * 3 + 2]);
            }
        }
    } else
        return NULL;

    free(lpbBuf);
    return tmp;
}

int Init_Font_Engine(int n_New_Width, int n_New_Height)
{
    if((fonts = Load_TrueColorBMP("Font.bmp")) == NULL)
        return 0;
    // nahraje bitmapu s fontem
    
    n_Width = n_New_Width;
    n_Height = n_New_Height;
    // nastav rozmry obrazovky

    return 1;
}

void Terminate_Font_Engine()
{
    if(fonts != NULL)
        free((void*)fonts);
}

void CharOut(int x, int y, char c, unsigned __int32 *vram)
{
    if(fonts == NULL)
        return;

    int cx, cy, nx, ny, cy1, y1;
    int z;

    z = c - 32;

    cx = c & 0x0f;
    cy = z / 16;

    cx = (cx * 11) & 0xff;
    cy = (cy * 13) & 0xff;

	cy1 = cy;
	y1 = y;
    for(nx = 0; nx < 11; nx ++, x ++, cx ++) {
		y = y1;
		cy = cy1;
        for(ny = 0; ny < 13; ny ++, y ++, cy ++) {
            if(x < n_Width && y < n_Height && x > 0 && y > 0) {
                if(fonts->bytes[cx + fonts->n_Width * cy] != 0) {
                    vram[x + n_Width * y] =
						fonts->bytes[cx + fonts->n_Width * cy];
                }
            }
        }
    }
}

void String_Out(char *str, int x, int y, unsigned __int32 *vram)
{
    int i, j;

    for(i = 0, j = 0; str[i] != 0; j ++, i ++) {
        if(str[i] == '\n') {
            y += 14;
            j = -1;
            continue;
        }
        CharOut(x + 11 * j, y, str[i], vram);
    }
}

void CharOut_NoMask(int x, int y, char c, unsigned __int32 *vram)
{
    if(fonts == NULL)
        return;

    int cx, cy, nx, ny, cy1, y1;
    int z;

    z = c - 32;

    cx = c & 0x0f;
    cy = z / 16;

    cx = (cx * 11) & 0xff;
    cy = (cy * 13) & 0xff;

	cy1 = cy;
	y1 = y;
    for(nx = 0; nx < 11; nx ++, x ++, cx ++) {
		y = y1;
		cy = cy1;
        for(ny = 0; ny < 13; ny ++, y ++, cy ++) {
            if(x < n_Width && y < n_Height && x > 0 && y > 0) {
                vram[x + n_Width * y] =
                    fonts->bytes[cx + fonts->n_Width * cy];
            }
        }
    }
}

void String_Out_NoMask(char *str, int x, int y, unsigned __int32 *vram)
{
    int i, j;

    for(i = 0, j = 0; str[i] != 0; j ++, i ++) {
        if(str[i] == '\n') {
            y += 14;
            j = -1;
            continue;
        }
        CharOut_NoMask(x + 11 * j, y, str[i], vram);
    }
}

void CharOut_AddTr(int x, int y, char c, unsigned __int32 *vram)
{
    if(fonts == NULL)
        return;

    int cx, cy, nx, ny, cy1, y1;
	unsigned __int32 a, t;
	int r, g, b;
    int z;

    z = c - 32;

    cx = c & 0x0f;
    cy = z / 16;

    cx = (cx * 11) & 0xff;
    cy = (cy * 13) & 0xff;

	cy1 = cy;
	y1 = y;
    for(nx = 0; nx < 11; nx ++, x ++, cx ++) {
		y = y1;
		cy = cy1;
        for(ny = 0; ny < 13; ny ++, y ++, cy ++) {
            if(x < n_Width && y < n_Height && x > 0 && y > 0) {
				a = fonts->bytes[cx + fonts->n_Width * cy];
				t = vram[x + n_Width * y];
				//
				r = (a & 0xff0000) + (t & 0xff0000);
				g = (a & 0x00ff00) + (t & 0x00ff00);
				b = (a & 0x0000ff) + (t & 0x0000ff);
				// secte rgb hodnoty
				
				r = (r > 0xff0000)? 0xff0000 : r & 0xff0000;
				g = (g > 0x00ff00)? 0x00ff00 : g & 0x00ff00;
				b = (b > 0x0000ff)? 0x0000ff : b & 0x0000ff;
				// a zkontroluje jestli nejsou moc vysoke
				// nebo je upravi do RGB masky

                vram[x + n_Width * y] = r | g | b;
            }
        }
    }
}

void String_Out_AddTr(char *str, int x, int y, unsigned __int32 *vram)
{
    int i, j;

    for(i = 0, j = 0; str[i] != 0; j ++, i ++) {
        if(str[i] == '\n') {
            y += 14;
            j = -1;
            continue;
        }
        CharOut_AddTr(x + 11 * j, y, str[i], vram);
    }
}